package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.type.JsonLongSetTypeHandler;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * OA 项目申请 DO
 *
 * @author 符溶馨
 */
@TableName(value ="bpm_oa_project", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAProjectDO extends BaseDO {

    /**
     * 出差表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 项目类型
     */
    private Integer type;

    /**
     * 项目名称
     */
    private String name;

    /**
     * 责任部门
     */
    private Long responsibleDept;

    /**
     * 参与部门
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> participationDept;

    /**
     * 责任人
     */
    private Long directorUserId;

    /**
     * 参与人员
     */
    @TableField(typeHandler = JsonLongSetTypeHandler.class)
    private Set<Long> staff;

    /**
     * 项目开始日期
     */
    private LocalDate startDate;

    /**
     * 项目结束日期
     */
    private LocalDate endDate;

    /**
     * 是否为长期项目 | 0否 1是
     */
    private Integer isLongTerm;

    /**
     * 项目预算
     */
    private BigDecimal projectBudget;

    /**
     * 项目内容
     */
    private String content;

    /**
     * 项目额外属性
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private Map<String, Object> dynamicAttribute;

    /**
     * 结果
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems;
}
