package cn.iocoder.yudao.module.crm.service.crmcontract;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.crm.controller.admin.crmcontract.vo.CheckInfoVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmcontract.vo.CrmContractPageReqVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmcontract.vo.CrmContractRespVO;
import cn.iocoder.yudao.module.crm.controller.admin.crmcontract.vo.CrmContractSaveReqVO;
import cn.iocoder.yudao.module.crm.dal.dataobject.crmcontract.CrmContractProductDO;
import cn.iocoder.yudao.module.hrm.api.crmcontract.dto.CrmContractProductSettlementDTO;
import cn.iocoder.yudao.module.product.api.storeproductattrvalue.vo.UserProductCountVO;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 合同 Service 接口
 *
 * @author yshop
 */
public interface CrmContractService {

    /**
     * 创建合同
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createContract(@Valid CrmContractSaveReqVO createReqVO);

    /**
     * 更新合同
     *
     * @param updateReqVO 更新信息
     */
    void updateContract(@Valid CrmContractSaveReqVO updateReqVO);

    /**
     * 删除合同
     *
     * @param id 编号
     */
    void deleteContract(Long id);

    /**
     * 获得合同
     *
     * @param id 编号
     * @return 合同
     */
    CrmContractRespVO getContract(Long id);

    /**
     * 获得合同分页
     *
     * @param pageReqVO 分页查询
     * @return 合同分页
     */
    PageResult<CrmContractRespVO> getContractPage(CrmContractPageReqVO pageReqVO);

    // ==================== 子表（合同产品关系） ====================

    /**
     * 获得合同产品关系列表
     *
     * @param contractId 合同ID
     * @return 合同产品关系列表
     */
    List<CrmContractProductDO> getContractProductListByContractId(Long contractId);

    /**
     * 生成合同编号
     *
     * @return
     */
    String getCode();

    /**
     * 审核合同
     *
     * @param checkInfoVO
     */
    void check(CheckInfoVO checkInfoVO);

    /**
     * 创建合同产品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    void createProduct(List<CrmContractProductDO> createReqVO);

    /**
     * 获取用户合同产品数量
     *
     * @param userIds
     * @param createTime
     * @return
     */
    List<UserProductCountVO> getProductCount(List<Long> userIds, LocalDateTime[] createTime);

    /**
     * 获取合同结算信息
     *
     * @return
     */
    List<CrmContractProductSettlementDTO> getContractProductSettlement();
}
