package cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaybalancesetting;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;
import java.util.*;
import java.time.LocalDateTime;
import java.time.LocalDateTime;
import java.time.LocalDateTime;
import com.baomidou.mybatisplus.annotation.*;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;

/**
 * 假期余额设置 DO
 *
 * @author 艾楷
 */
@TableName("kq_holiday_balance_setting")
@KeySequence("kq_holiday_balance_setting_seq") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class HolidayBalanceSettingDO extends BaseDO {

    /**
     * 编号
     */
    @TableId
    private Long id;
    /**
     * 假期设置id
     */
    private Long holidaySettingId;
    /**
     * 状态 0关闭 1开启
     */
    private Integer status;
    /**
     * 发放方式 1每月自动发放 2每年自动发放 3手动发放 4加班时长自动计入余额
     */
    private Integer type;
    /**
     * 发放日期 当发放方式为每月自动发放时 该值代表的是几号 当发放方式为每年自动发放时 该值代表 1每年员工入职日 2每年1月1日
     */
    private Integer issueTimeType;
    /**
     * 发放日期(设置好后 插入发放日期 - 发放完成后插入下一次发放日期) 如果是 null 的话代表手动发放和加班自动计算或者每年员工入职日或者每年员工入职日
     */
    private LocalDateTime issueTime;
    /**
     * 额度规则 1固定规则 2按工龄 （0代表自动发放或者加班自动计算）
     */
    private Integer quotaRule;
    /**
     * 固定额度 （单位天）
     */
    private Integer quota;
    /**
     * 是否按实际工作时长发放 0否 1是(例如：每年1月1日发放6天假期，入职时间是6月1日的员工则按照半年发放3天假期，当余额出现小数时按四舍五入取整)
     */
    private Integer actualWorkFlag;
    /**
     * 有效期类型 1自发放日起1个月 2自发放日起1周年 3按入职日期起12个月 4自然年1月1日-12月31日 5每年固定时间作废 6永久有效 7每月固定时间作废 8每季末作废 9加班多少天后作废
     */
    private Integer validityPeriod;
    /**
     * 加班多少天后作废
     */
    private Integer afterFewDays;
    /**
     * 每年固定时间作废格式 MM-dd 例如 09-10 每年9月10号作废
     */
    private String fixedEveryYearInvalid;
    /**
     * 每月固定时间作废 格式 1 or 4 or 7 or 10
     */
    private Integer fixedEveryMonthInvalid;
    /**
     * 是否允许延长有效期 0否 1是
     */
    private Integer extensionAllowedFlag;
    /**
     * 有效期过后延长时间（天）
     */
    private Integer extendNum;
    /**
     * 是否开启假期提醒 0否 1是
     */
    private Integer expirationReminderFlag;
    /**
     * 提醒时间（天） 假期过期前多少天提醒
     */
    private Integer reminderTimeNum;
    /**
     * 提醒范围 1员工 2部门主管 3全部通知
     */
    private Integer reminderRange;

    public HolidayBalanceSettingDO init(Long holidaySettingId) {
        this.holidaySettingId = holidaySettingId;
        this.status = 0;
        this.type = 1;
        this.issueTimeType = 1;
        this.quotaRule = 1;
        this.quota = 5;
        this.validityPeriod = 1;
        return this;
    }
}
