package cn.iocoder.yudao.module.system.service.holiday.holidaybalancesetting;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidaybalancesetting.vo.HolidayBalanceSettingPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidaybalancesetting.vo.HolidayBalanceSettingSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidaybalancesetting.HolidayBalanceSettingDO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * 假期余额设置 Service 接口
 *
 * @author 艾楷
 */
public interface HolidayBalanceSettingService {

    /**
     * 创建假期余额设置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createHolidayBalanceSetting(@Valid HolidayBalanceSettingSaveReqVO createReqVO);

    /**
     * 更新假期余额设置
     *
     * @param updateReqVO 更新信息
     */
    void updateHolidayBalanceSetting(@Valid HolidayBalanceSettingSaveReqVO updateReqVO);

    /**
     * 删除假期余额设置
     *
     * @param id 编号
     */
    void deleteHolidayBalanceSetting(Long id);

    /**
     * 获得假期余额设置
     *
     * @param id 编号
     * @return 假期余额设置
     */
    HolidayBalanceSettingDO getHolidayBalanceSetting(Long id);

    /**
     * 获得假期余额设置分页
     *
     * @param pageReqVO 分页查询
     * @return 假期余额设置分页
     */
    PageResult<HolidayBalanceSettingDO> getHolidayBalanceSettingPage(HolidayBalanceSettingPageReqVO pageReqVO);

    /**
     * 通过及其设置id查询假期余额设置
     *
     * @param settingId
     * @return
     */
    HolidayBalanceSettingDO selectBySettingId(Long settingId);

    /**
     * 通过及其设置ids查询假期余额设置
     *
     * @param settingIds
     * @return
     */
    Map<Long, HolidayBalanceSettingDO> selectBySettingIds(List<Long> settingIds);

    /**
     * 通过假期设置ids获取假期余额设置列表
     *
     * @param holidaySettingIds
     * @return
     */
    List<HolidayBalanceSettingDO> getHolidayBalanceSettingByHolidaySettingIds(List<Long> holidaySettingIds);

    /**
     * 获取假期余额设置
     *
     * @return
     */
    List<HolidayBalanceSettingDO> list(LambdaQueryWrapper<HolidayBalanceSettingDO> lambdaQueryWrapper);

    /**
     * 批量更新
     *
     * @param editList
     */
    void batchUpdate(List<HolidayBalanceSettingDO> editList);

    /**
     * 根据ids获取
     * @param holidayBalanceSettingIds
     * @return
     */
    List<HolidayBalanceSettingDO> getByIds(List<Long> holidayBalanceSettingIds);
}
