package cn.iocoder.yudao.module.system.service.hr.type;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.system.controller.admin.hr.vo.type.SalaryTypeItemCreateReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.hr.SalaryTypeItemDO;
import cn.iocoder.yudao.module.system.dal.mysql.hr.SalaryTypeItemMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

@Service
public class SalaryTypeItemServiceImpl implements SalaryTypeItemService{

    @Resource
    private SalaryTypeItemMapper salaryTypeItemMapper;

    @Override
    public Long createSalaryTypeItem(SalaryTypeItemCreateReqVO createReqVO) {

        // 校验code的唯一性
        validateTypeCodeExists(null, createReqVO.getCode());

        SalaryTypeItemDO itemDO = BeanUtils.toBean(createReqVO, SalaryTypeItemDO.class);
        itemDO.setStatus(CommonStatusEnum.ENABLE.getStatus());

        // 插入
        salaryTypeItemMapper.insert(itemDO);
        return itemDO.getId();
    }

    @Override
    public void updateSalaryTypeItem(SalaryTypeItemCreateReqVO updateReqVO) {

        validateTypeItemExists(updateReqVO.getId());
        // 校验code的唯一性
        validateTypeCodeExists(updateReqVO.getId(), updateReqVO.getCode());

        SalaryTypeItemDO updateObj = BeanUtils.toBean(updateReqVO, SalaryTypeItemDO.class);
        salaryTypeItemMapper.updateById(updateObj);
    }

    @Override
    public void updateSalaryTypeItemStatus(Long id, Integer status) {

        validateTypeItemExists(id);

        SalaryTypeItemDO itemDO = new SalaryTypeItemDO()
                .setId(id)
                .setStatus(status);

        salaryTypeItemMapper.updateById(itemDO);
    }

    @Override
    public void updateStatusByTypeId(Long typeId, Integer status) {

        salaryTypeItemMapper.update(new SalaryTypeItemDO().setStatus(status), new LambdaQueryWrapperX<SalaryTypeItemDO>()
                .eq(SalaryTypeItemDO::getTypeId, typeId));
    }

    @Override
    public SalaryTypeItemDO getSalaryTypeItem(Long id) {

        return salaryTypeItemMapper.selectById(id);
    }

    @Override
    public List<SalaryTypeItemDO> getListByTypeId(Long typeId) {

        return salaryTypeItemMapper.selectList(SalaryTypeItemDO::getTypeId, typeId);
    }

    private void validateTypeItemExists(Long id) {
        if (salaryTypeItemMapper.selectById(id) == null) {
            throw exception(SALARY_TYPE_ITEM_NOT_EXISTS);
        }
    }

    private void validateTypeCodeExists(Long id, String code) {

        if (StrUtil.isBlank(code)) {
            return;
        }

        // 校验类型名称唯一性
        SalaryTypeItemDO itemDO = salaryTypeItemMapper.selectOne(SalaryTypeItemDO::getCode, code);
        if (itemDO == null) {
            return;
        }
        if (id == null) {
            throw exception(SALARY_TYPE_ITEM_CODE_EXISTS);
        }
        if (!itemDO.getId().equals(id)) {
            throw exception(SALARY_TYPE_ITEM_CODE_EXISTS);
        }
    }
}
