package cn.iocoder.yudao.module.system.service.rental;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.rentalsalerecord.RentalSaleRecordPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.rentalsalerecord.RentalSaleRecordRespVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.rentalsalerecord.RentalSaleRecordSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.rentalsalerecord.RentalSaleStatusReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.rental.RentalSaleRecordDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 租赁转销售记录 Service 接口
 *
 * @author 符溶馨
 */
public interface RentalSaleRecordService {

    /**
     * 创建租赁转销售记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createRentalSaleRecord(@Valid RentalSaleRecordSaveReqVO createReqVO);

    /**
     * 创建租赁转销售记录
     *
     * @param createReqVO 创建信息
     */
    void createBatch(@Valid List<RentalSaleRecordSaveReqVO> createReqVO);

    /**
     * 更新租赁转销售记录
     *
     * @param updateReqVO 更新信息
     */
    void updateRentalSaleRecord(@Valid RentalSaleStatusReqVO updateReqVO);

    /**
     * 删除租赁转销售记录
     *
     * @param id 编号
     */
    void deleteRentalSaleRecord(Long id);

    /**
     * 获得租赁转销售记录
     *
     * @param id 编号
     * @return 租赁转销售记录
     */
    RentalSaleRecordDO getRentalSaleRecord(Long id);

    /**
     * 获得租赁转销售记录分页
     *
     * @param pageReqVO 分页查询
     * @return 租赁转销售记录分页
     */
    PageResult<RentalSaleRecordDO> getRentalSaleRecordPage(RentalSaleRecordPageReqVO pageReqVO);

    /**
     * 获取所有到达周转期的租赁订单押金记录
     * @return 订单押金记录
     */
    List<RentalSaleRecordRespVO> getSaleDepositRecordList();
}